// File: Concordance.java
// Count how many times each character appears in a file. 
//  For simplicity, letters are counted case insensitive
// Written by Dr. Spiegel

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

public class Concordance {

  public static final Integer ONE = new Integer(1);


  // Read the input file name from the user
  public static String GetInputFile() throws java.io.IOException
  { DataInputStream FileNameInput=new DataInputStream(System.in);
    System.out.print("File to Open >");
    System.out.flush();
    return(FileNameInput.readLine());
  }

  // Update the character occurance counters
  static void ProcessLine(Map CC,String Line)
  { for (int i=0;i<Line.length();i++) {
      char thisChar=Line.charAt(i);
      Integer Value = (Integer) CC.get(thisChar);
      // Increment number of occurrences of this char
      if (Value == null) {
        CC.put(thisChar, ONE);
      } 
      else {  // increment count
        CC.put(thisChar, new Integer(Value.intValue()+1));
      }
    }
  }


  // Show the number of occurances of each character in the file
  static void ShowCharacterCountChart(Map CharCount)
                                       //         throws java.io.IOException
  { // Print values of map, one per line
    Iterator it = CharCount.keySet().iterator();
    DecimalFormat Form=new DecimalFormat("##0");

    int count=0;
    System.out.println("Ordered by keys");
    while (it.hasNext()) {
      Object key = it.next();
      // Use a StringBuffer to set desired length
      StringBuffer S=new StringBuffer(Form.format(CharCount.get(key)));
      S.setLength(4);
      System.out.print(key + " = " + S);
      if (count++%8==7)  // make it pretty
	System.out.println();
    }
    System.out.println();

    count=0;
    System.out.println("Ordered by keys, version 2");
    it = CharCount.entrySet().iterator();
    while (it.hasNext() ) {
      Map.Entry e = (Map.Entry) it.next();
      System.out.print(e.getKey() + " = " + e.getValue()+" ");
      if (count++%10==9)  // make it pretty
	System.out.println();
    }
    System.out.println();

    count=0;
    System.out.println("Letter Counts Only");
    // Place the upper case letters in a List
    ArrayList<Character> Letters=new ArrayList();
    for (Character c='A';c<='Z';c++)
      Letters.add(c);
    for (char ch:Letters) {
      // must check if there were any occurrences of that letter first!
      // Could also do this when building the collection Letters
      if (CharCount.get(ch) != null) {
        Object key = new Character(ch);;
        // Use a StringBuffer to set desired length
        StringBuffer S=new StringBuffer(Form.format(CharCount.get(key)));
        S.setLength(4);
        System.out.print(key + " = " + S);
        if (count++%8==7)  // make it pretty
	  System.out.println();
      }
    }  // last for
  } // ShowCharacterCount

 public static void main(String[] args) throws java.io.IOException
 { // Associate the disk file with the File object
   File InputFile=new File(GetInputFile());
   // Now, open a stream from the input file
   DataInputStream DataInput=new DataInputStream(new FileInputStream(InputFile));
   // Read the file one line at a time and process each line
   String Line;

   // Use a TreeMap to keep count (why can't it be <char,int>?)
   Map<Character,Integer> CharCount=new TreeMap();

   while ((Line=DataInput.readLine())!=null)
     ProcessLine(CharCount,Line.toUpperCase());
   System.out.println("Map print en masse:\n"+CharCount);
   ShowCharacterCountChart(CharCount);
 }


}

